
from otherstuf import *

def test_chainstuf():
    """Test chainstuf class"""
    
    # make some base dicts
    d1 = dict(this=1, that=2)
    d2 = dict(roger=99, that=100)
    
    # test simple attribute equivalence
    dd = chainstuf(d1, d2)
    assert dd.this == 1
    assert dd.roger == 99
    assert dd.this == dd['this']
    assert dd.that == dd['that']
    assert dd.roger == dd['roger']
    
    # set value on chainstuf, ensure properly set, in top dict
    dd.roger = 'wilco'
    assert dd.roger == 'wilco'
    assert dd.roger == d1['roger']
    
    # test new_child
    dd2 = dd.new_child()
    dd2.smorg = 44
    assert dd2.smorg == 44
    dd.roger = 'roger'
    assert dd2.roger == 'roger'
    
def test_counterstuf():
    """Test counterstuf class"""
    c = counterstuf()
    c.update("this and this is this but that isn't this".split())
    c.total = sum(c.values())
    assert c.total == 9
    assert c.this == 4
    assert c.but == 1
    assert c.bozo == 0
    c2 = counterstuf().update_self("big big small big medium xlarge".split())
    assert c2.medium == 1
    assert dict(c2.most_common()) == dict([('big', 3), ('small', 1), ('medium', 1), ('xlarge', 1)])



"""
Sidecar to `stuf` that adds `ChainMap`-like 
container `chainstuf`
"""

try:
    from collections import ChainMap
except ImportError:
    from stuf.collects import ChainMap


class chainstuf(ChainMap):
    """
    A stuf-like surfacing of the ChainMap collection
    (multi-layer dict) introduced in Python 3. Uses a
    workalike replacement under Python 2.
    """
    def __init__(self, *maps):
        ChainMap.__init__(self, *maps)
        
    def __getattr__(self, key):
        # handle normal object attributes
        if key in self.__dict__:
            return self.__dict__[key]
        # handle special attributes
        else:
            for m in self.maps:
                try:
                    return m[key]
                except KeyError:
                    pass
            raise KeyError(key)
    
    def __setattr__(self, key, value):
        # handle normal object attributes
        if key == 'maps' or key in self.__dict__:
            ChainMap.__setattr__(self, key, value)
        else:
            self.maps[0][key] = value



"""
Sidecar to `stuf` that adds `Counter`-like 
container `counterstuf`
"""
try:
    from collections import Counter
except ImportError:
    from stuf.collects import Counter

class counterstuf(Counter):
    """stuf-like surfacing of Counter"""
    
    def __getattr__(self, key):
        if key in self.__dict__:
            return self.__dict__[key]
        else:
            return self[key]

    def __setattr__(self, key, value):
        if key in self.__dict__:
            self.__dict__[key] = value
        else:
            self[key] = value
        
    def update_self(self, *args, **kwargs):
        self.update(*args, **kwargs)
        return self
    
    def copy(self):
        return counterstuf(Counter.copy(self))



#! /usr/bin/env python

from setuptools import setup
from decimal import Decimal
import re

def linelist(text):
    """
    Returns each non-blank line in text enclosed in a list.
    """
    return [ l.strip() for l in text.strip().splitlines() if l.split() ]
    
    # The double-mention of l.strip() is yet another fine example of why
    # Python needs en passant aliasing.


def verno(s):
    """
    Update the version number passed in by extending it to the 
    thousands place and adding 1/1000, then returning that result
    and as a side-effect updating setup.py

    Dangerous, self-modifying, and also, helps keep version numbers
    ascending without human intervention.
    """
    d = Decimal(s)
    increment = Decimal('0.001')
    d = d.quantize(increment) + increment
    dstr = str(d)
    setup = open('setup.py', 'r').read()
    setup = re.sub('verno\(\w*[\'"]([\d\.]+)[\'"]', 'verno("' + dstr + '"', setup)
    open('setup.py', 'w').write(setup)
    return dstr

setup(
    name='otherstuf',
    version=verno("0.802"),
    author='Jonathan Eunice',
    author_email='jonathan.eunice@gmail.com',
    description='Attributes-accessible mappings chainstuf (like ChainMap) & counterstuf (like Counter)',
    long_description=open('README.rst').read(),
    url='https://bitbucket.org/jeunice/otherstuf',
    packages=['otherstuf'],
    install_requires=['stuf>=0.9.10'],
    tests_require = ['tox', 'pytest'],
    zip_safe = True,
    keywords='Counter ChainMap stuf attribute mapping nested',
    classifiers=linelist("""
        Development Status :: 4 - Beta
        Operating System :: OS Independent
        License :: OSI Approved :: BSD License
        Intended Audience :: Developers
        Programming Language :: Python
        Programming Language :: Python :: 2.6
        Programming Language :: Python :: 2.7
        Programming Language :: Python :: 3.2
        Programming Language :: Python :: 3.3
        Programming Language :: Python :: Implementation :: CPython
        Programming Language :: Python :: Implementation :: PyPy
        Topic :: Software Development :: Libraries :: Python Modules
    """)
)
